/* 
 * Piotr 'Deletek' Kupisiewicz - AutoWRR 
 * 
 * Description:
 * This program should be some kind of wizard for setting up
 * WRR scheulder
 *
 * Homepage: 
 * http://www.rzepaknet.us/~deletek/autowrr/
 */
 
#include <cdk/cdk.h>

#ifdef HAVE_XCURSES
char *XCursesProgramName="autowrr-wizard";
#endif

/* Declare variables */
int selection;
char *name_in;
char *name_out;
int band_in;
int band_out;
int band_ports;
char HTB;
char *ports;
char *proxy;

/* Declare objects */
CDKBUTTONBOX *buttonWidget	= 0;
CDKDIALOG *dialog_welcome	= 0;
CDKDIALOG *dialog_htb		= 0;
CDKDIALOG *dialog_confirm	= 0;
CDKENTRY *entry_in		= 0;
CDKENTRY *entry_out		= 0;
CDKENTRY *entry_bwin		= 0;
CDKENTRY *entry_bwout		= 0;
CDKENTRY *entry_bwports		= 0;
CDKENTRY *entry_ports		= 0;
CDKENTRY *entry_proxy		= 0;

/* Declare screens */
CDKSCREEN *cdkscreen		= 0;
WINDOW *cursesWin		= 0;

/* Function used to switch Buttons */
static BINDFN_PROTO(entryCB);
/* Cleaning up function */
void end(void);

/* Help function */
void help(void)
{
	printf("Properly usage: \n");
	printf("\tautowrr <file_name>\n");
	printf("\tWhere <file_name> is name output file\n");
}

void too_small(void)
{	
	end();
	printf("Window is too small. Make it bigger\n");
	exit(1);
}

/* Function that initializes graphics and whole CDK enviromenment */
int initialize(void)
{
	/* Set up CDK. */
	cursesWin = initscr();
	cdkscreen = initCDKScreen (cursesWin);
	
	/* Start color. */
	initCDKColor();
   
	/* Create the entry widget. */
	entry_in = newCDKEntry (cdkscreen, CENTER, CENTER,
		"<C>Enter name of the IN interface (from the LAN side)", "Interface name: ", A_NORMAL, '.', vMIXED,
		40, 0, 256, TRUE, FALSE);
	setCDKEntryLLChar (entry_in, ACS_LTEE);
	setCDKEntryLRChar (entry_in, ACS_RTEE);
   	
	entry_out = newCDKEntry (cdkscreen, CENTER, CENTER,
		"<C>Enter name of the OUT interface (for instance from the Internet side)", "Interface name: ", 
		A_NORMAL, '.', vMIXED, 40, 0, 256, TRUE, FALSE);
	setCDKEntryLLChar (entry_out, ACS_LTEE);
	setCDKEntryLRChar (entry_out, ACS_RTEE);

	entry_bwin = newCDKEntry (cdkscreen, CENTER, CENTER,
		"<C>Enter the maximum IN bandwitch (from the LAN side) - in Kbits", "Value: ", A_NORMAL, '.',
		vINT, 40, 0, 256, TRUE, FALSE);
	setCDKEntryLLChar (entry_bwin, ACS_LTEE);
	setCDKEntryLRChar (entry_bwin, ACS_RTEE);
   
	entry_bwout = newCDKEntry (cdkscreen, CENTER, CENTER,
		"<C>Enter the maximum OUT bandwitch (for instance from the Internet side)", "Value: ",
		 A_NORMAL, '.', vINT, 40, 0, 256, TRUE, FALSE);
	setCDKEntryLLChar (entry_bwout, ACS_LTEE);
	setCDKEntryLRChar (entry_bwout, ACS_RTEE);

	entry_bwports = newCDKEntry (cdkscreen, CENTER, CENTER,
		"<C>Enter the maximum bandwitch for special ports (selected in next page)",
		 "Value: ", A_NORMAL, '.', vINT, 40, 0, 256, TRUE, FALSE);
	setCDKEntryLLChar (entry_bwports, ACS_LTEE);
	setCDKEntryLRChar (entry_bwports, ACS_RTEE);

	entry_ports = newCDKEntry (cdkscreen, CENTER, CENTER,
		"<C>Enter the ports separated by SPACE (without commons or fullstops) to be included in priority class", 
		"Ports: ", A_NORMAL, '.', vMIXED, 40, 0, 256, TRUE, FALSE);
	setCDKEntryLLChar (entry_ports, ACS_LTEE);
	setCDKEntryLRChar (entry_ports, ACS_RTEE);

	entry_proxy = newCDKEntry (cdkscreen, CENTER, CENTER,
		"<C>Enter ports used by proxy (if you use the transparent proxy enter 80 also here) - see documentation", 
		"Ports: ", A_NORMAL, '.', vMIXED, 40, 0, 256, TRUE, FALSE);
	setCDKEntryLLChar (entry_proxy, ACS_LTEE);
	setCDKEntryLRChar (entry_proxy, ACS_RTEE);

	/*
 	 * Check if the window isn't too small too display it properly
	*/
   	if(entry_in == 0 || entry_out == 0 || entry_bwin == 0 || entry_bwout == 0 || entry_bwports == 0 || 
		entry_ports == 0)
		too_small();
}

void welcome(void)
{

	char *message[9]; 
	char *buttons[] = { "OK" };

	/* Erase CDK. */
	eraseCDKScreen (cdkscreen);

   
	message[0]   = "<C></U>AutoWRR by Deletek";
	message[1]   = " ";
	message[2]   = "This program should be some kind of wizard";
	message[3]   = "to help you setting up WRR scheulder.";
	message[4]   = " ";
	message[5]   = "<C>I hope it can be helpful :-)";
	message[6]   = " ";
	message[7]   = "Contact with me: deli@rzepaknet.us";
	message[8]  = "Homepage: www.rzepaknet.us/~deletek/autowrr";
    
	dialog_welcome = newCDKDialog (cdkscreen, CENTER, CENTER, message, 9, buttons, 1, COLOR_PAIR(2)|A_REVERSE,
		TRUE, TRUE, FALSE);

	/*
	 * Check if the window isn't too small to display it properly
 	*/
	if(dialog_welcome == 0)
		too_small();									       

	activateCDKDialog(dialog_welcome, 0);

}
/* Function that asks us about an Incoming Interface */
int interface_in(void)
{
	char *buttons[]		= {" OK ", " Exit "};

	/* Erase CDK. */
	eraseCDKScreen (cdkscreen);

	/* Create the button box widget. */
	buttonWidget = newCDKButtonbox (cdkscreen,
		getbegx(entry_in->win),
		getbegy(entry_in->win) + entry_in->boxHeight - 1,
		1, entry_in->boxWidth - 1,
		0, 1, 2,
		buttons, 2, A_REVERSE,
		TRUE, FALSE);

	/* Set the lower left and right characters of the box. */
	setCDKButtonboxULChar (buttonWidget, ACS_LTEE);
	setCDKButtonboxURChar (buttonWidget, ACS_RTEE);

	  /*
	   * Bind the Tab key in the entry field to send a
	   * Tab key to the button box widget.
	   */
	   bindCDKObject (vENTRY, entry_in, KEY_TAB, entryCB, buttonWidget);

	   /* Activate the entry field. */
	   drawCDKButtonbox (buttonWidget, TRUE);
	   name_in = copyChar (activateCDKEntry (entry_in, 0));

	   /* Clean up. */
	   return buttonWidget->currentButton;
}

/* Function that asks us about an Outgoing Interface */
int interface_out(void)
{
	char *buttons[]		= {" OK ", " Back "};
	
	/* Erase CDK. */
	eraseCDKScreen (cdkscreen);

	/* Create the button box widget. */
	buttonWidget = newCDKButtonbox (cdkscreen,
		getbegx(entry_out->win),
		getbegy(entry_out->win) + entry_out->boxHeight - 1,
		1, entry_out->boxWidth - 1,
		0, 1, 2,
		buttons, 2, A_REVERSE,
		TRUE, FALSE);

	/* Set the lower left and right characters of the box. */
	setCDKButtonboxULChar (buttonWidget, ACS_LTEE);
	setCDKButtonboxURChar (buttonWidget, ACS_RTEE);

	/*
	 * Bind the Tab key in the entry field to send a
	 * Tab key to the button box widget.
	*/
	bindCDKObject (vENTRY, entry_out, KEY_TAB, entryCB, buttonWidget);

	/* Activate the entry field. */
	drawCDKButtonbox (buttonWidget, TRUE);
	name_out = copyChar (activateCDKEntry (entry_out, 0));

	/* Clean up. */
	return buttonWidget->currentButton;
}

/* Function that asks us about a bandwitch of Incoming Interface */
int bandwitch_in(void)
{
	char *buttons[]		= {" OK ", " Back "};

	/* Erase CDK. */
	eraseCDKScreen (cdkscreen);

	/* Create the button box widget. */
	buttonWidget = newCDKButtonbox (cdkscreen,
		getbegx(entry_bwin->win),
		getbegy(entry_bwin->win) + entry_bwin->boxHeight - 1,
		1, entry_bwin->boxWidth - 1,
		0, 1, 2,
		buttons, 2, A_REVERSE,
		TRUE, FALSE);

	/* Set the lower left and right characters of the box. */
	setCDKButtonboxULChar (buttonWidget, ACS_LTEE);
	setCDKButtonboxURChar (buttonWidget, ACS_RTEE);

	/*
	 * Bind the Tab key in the entry field to send a
	 * Tab key to the button box widget.
	*/
	bindCDKObject (vENTRY, entry_bwin, KEY_TAB, entryCB, buttonWidget);

	/* Activate the entry field. */
	drawCDKButtonbox (buttonWidget, TRUE);
	band_in = atoi(activateCDKEntry (entry_bwin, 0));

	/* Clean up. */
	return buttonWidget->currentButton;
}

/* Function that asks us about a bandwitch of Incoming Interface */
int bandwitch_out(void)
{
	char *buttons[]		= {" OK ", " Back "};

	/* Erase CDK. */
	eraseCDKScreen (cdkscreen);

	/* Create the button box widget. */
	buttonWidget = newCDKButtonbox (cdkscreen,
		getbegx(entry_bwout->win),
		getbegy(entry_bwout->win) + entry_bwout->boxHeight - 1,
		1, entry_bwout->boxWidth - 1,
		0, 1, 2,
		buttons, 2, A_REVERSE,
		TRUE, FALSE);

	/* Set the lower left and right characters of the box. */
	setCDKButtonboxULChar (buttonWidget, ACS_LTEE);
	setCDKButtonboxURChar (buttonWidget, ACS_RTEE);

	/*
	 * Bind the Tab key in the entry field to send a
	 * Tab key to the button box widget.
	*/
	bindCDKObject (vENTRY, entry_bwout, KEY_TAB, entryCB, buttonWidget);

	/* Activate the entry field. */
	drawCDKButtonbox (buttonWidget, TRUE);
	band_out = atoi (activateCDKEntry (entry_bwout, 0));

	/* Clean up. */
	return buttonWidget->currentButton;
}

int bandwitch_ports(void)
{
	char *buttons[]		= {" OK ", " Back "};

	/* Erase CDK. */
	eraseCDKScreen (cdkscreen);

	/* Create the button box widget. */
	buttonWidget = newCDKButtonbox (cdkscreen,
		getbegx(entry_bwports->win),
		getbegy(entry_bwports->win) + entry_bwports->boxHeight - 1,
		1, entry_bwports->boxWidth - 1,
		0, 1, 2,
		buttons, 2, A_REVERSE,
		TRUE, FALSE);

	/* Set the lower left and right characters of the box. */
	setCDKButtonboxULChar (buttonWidget, ACS_LTEE);
	setCDKButtonboxURChar (buttonWidget, ACS_RTEE);

	/*
	 * Bind the Tab key in the entry field to send a
	 * Tab key to the button box widget.
	*/
	bindCDKObject (vENTRY, entry_bwports, KEY_TAB, entryCB, buttonWidget);

	/* Activate the entry field. */
	drawCDKButtonbox (buttonWidget, TRUE);
	band_ports = atoi (activateCDKEntry (entry_bwports, 0));

	/* Clean up. */
	return buttonWidget->currentButton;
}

int get_ports(void)
{
	char *buttons[]		= {" OK ", " Back "};

	/* Erase CDK. */
	eraseCDKScreen (cdkscreen);

	/* Create the button box widget. */
	buttonWidget = newCDKButtonbox (cdkscreen,
		getbegx(entry_ports->win),
		getbegy(entry_ports->win) + entry_ports->boxHeight - 1,
		1, entry_ports->boxWidth - 1,
		0, 1, 2,
		buttons, 2, A_REVERSE,
		TRUE, FALSE);

	/* Set the lower left and right characters of the box. */
	setCDKButtonboxULChar (buttonWidget, ACS_LTEE);
	setCDKButtonboxURChar (buttonWidget, ACS_RTEE);

	/*
	 * Bind the Tab key in the entry field to send a
	 * Tab key to the button box widget.
	*/
	bindCDKObject (vENTRY, entry_ports, KEY_TAB, entryCB, buttonWidget);

	/* Activate the entry field. */
	drawCDKButtonbox (buttonWidget, TRUE);
	ports = copyChar (activateCDKEntry (entry_ports, 0));

	/* Clean up. */
	return buttonWidget->currentButton;
}

/* Function asking about HTB or CBQ using */
int htb(void)
{
	char *message[3]; 
	int selection;
	char *buttons[] = { " CBQ " , " HTB ", " Back "};

	/* Erase CDK. */
	eraseCDKScreen (cdkscreen);
   
	message[0]   = "<C></U>Now you should choose traffic controller";
	message[1]   = "CBQ is recommended because it is supported by more kernels ";
	message[2]   = "And it is also more precissaly.";
    
	dialog_htb = newCDKDialog (cdkscreen, CENTER, CENTER, message, 3, buttons, 3, COLOR_PAIR(2)|A_REVERSE,
		TRUE, TRUE, FALSE);

	/*
	 * Check if the window isn't too small to display it properly
 	*/
	if(dialog_htb == 0)
		too_small();									       

	selection = activateCDKDialog(dialog_htb, 0);
	if(selection == 0)
	    HTB = 'n';
	if(selection == 1)
	    HTB = 'y';
	if(selection == 2)
	    return 1;
	return 0;
}

int transparent_proxy(void)
{
	char *buttons[]		= {" OK ", " Back "};

	/* Erase CDK. */
	eraseCDKScreen (cdkscreen);

	/* Create the button box widget. */
	buttonWidget = newCDKButtonbox (cdkscreen,
		getbegx(entry_proxy->win),
		getbegy(entry_proxy->win) + entry_proxy->boxHeight - 1,
		1, entry_proxy->boxWidth - 1,
		0, 1, 2,
		buttons, 2, A_REVERSE,
		TRUE, FALSE);

	/* Set the lower left and right characters of the box. */
	setCDKButtonboxULChar (buttonWidget, ACS_LTEE);
	setCDKButtonboxURChar (buttonWidget, ACS_RTEE);

	/*
	 * Bind the Tab key in the entry field to send a
	 * Tab key to the button box widget.
	*/
	bindCDKObject (vENTRY, entry_proxy, KEY_TAB, entryCB, buttonWidget);

	/* Activate the entry field. */
	drawCDKButtonbox (buttonWidget, TRUE);
	proxy = copyChar (activateCDKEntry (entry_proxy, 0));

	/* Clean up. */
	return buttonWidget->currentButton;
}

int confirm(void)
{
	char *message[9]; 
	char *buttons[] = { "OK ", " Back "};
	char temp[100];

	/* Erase CDK. */
	eraseCDKScreen (cdkscreen);
   
	message[0]   = "<C>These are your settings, confirm:";
	sprintf(temp, "Device in (LAN SIDE): %s", name_in);
	message[1] = copyChar(temp);
	sprintf(temp, "Device out (WAN SIDE): %s", name_out);
	message[2] = copyChar(temp);
	sprintf(temp, "Bandwitch from IN side: %d", band_in);
	message[3] = copyChar(temp);
	sprintf(temp, "Bandwitch from OUT side: %d", band_out); 
	message[4] = copyChar(temp);
	sprintf(temp, "Bandwitch for special ports and ICMP: %d", band_ports);
	message[5] = copyChar(temp);
	if(ports != NULL && strlen(ports) != 0)
		sprintf(temp, "Special ports: \"%s\"", ports); 
	else
		sprintf(temp, "There are no special ports");
	message[6] = copyChar(temp);
	if(HTB == 'y')
	    sprintf(temp, "You want to use HTB");
	else
	    sprintf(temp, "You want to use CBQ");
	message[7] = copyChar(temp);
	if(proxy != NULL && strlen(proxy) != 0)
	    sprintf(temp, "Proxy ports: %s", proxy);
	else 
	    sprintf(temp, "You do not want to use proxy.");
    	message[8] = copyChar(temp);
	dialog_confirm = newCDKDialog (cdkscreen, CENTER, CENTER, message, 9, buttons, 2, COLOR_PAIR(2)|A_REVERSE,
		TRUE, TRUE, FALSE);

	/*
	 * Check if the window isn't too small to display it properly
 	*/
	if(dialog_confirm == 0)
		too_small();									       

	return  activateCDKDialog(dialog_confirm, 0);
}


/* Function that destroys all objects freeing memory */
void destroy_all(void)
{
	destroyCDKButtonbox (buttonWidget);
	destroyCDKDialog (dialog_welcome);
	destroyCDKDialog (dialog_htb);
	destroyCDKDialog (dialog_confirm);
	destroyCDKEntry (entry_in);
	destroyCDKEntry (entry_out);
	destroyCDKEntry (entry_bwin);
	destroyCDKEntry (entry_bwout);
	destroyCDKEntry (entry_bwports);
        destroyCDKEntry (entry_ports);
	destroyCDKEntry (entry_proxy);
}

/* Function that destroy CDK screen and ends CDK */
void end(void)
{
	destroyCDKScreen (cdkscreen);
	delwin (cursesWin);
	endCDK();
}


/* Function used to switch Buttons */
static int entryCB (EObjectType cdktype GCC_UNUSED, void *object GCC_UNUSED, void *clientData, chtype key)
{
   CDKBUTTONBOX *buttonbox = (CDKBUTTONBOX *)clientData;
   injectCDKButtonbox (buttonbox, key);
   return (TRUE);
}

/* And the main function ... */
int main (int argc, char *argv[])
{
	/* File descriptor */
	FILE *out;
    
	/* Check if the arguments count is properly */
	if(argc > 2 || argc == 1)
	{
		/* if not shows help */
		help(); 
		/* and exit :-) */
		exit(1);
	}
	
	/* Initializing graphs */
	initialize();

	/* Welcome Dialog */
	welcome();
    
	/* 
	 * Something that makes it work
	 * It doesn't look good, but works
	 * preety nice 
	 * Maybe someone can fix it
	 */
     
	if_in:
	if( interface_in() == 1)
	{
		end();
		exit(1);
	}
	if_out:
	selection = interface_out();
	if(selection == 1)
		goto if_in;
	band_in:
	selection = bandwitch_in();
	if(selection == 1)
		goto if_out;
	band_out:
	selection = bandwitch_out();
	if(selection == 1)
		goto band_in;
	band_ports:
	selection = bandwitch_ports();
	if(selection == 1)
		goto band_out;
	get_ports:
	selection = get_ports();
	if(selection == 1)
		goto band_ports;
	htb:
	selection = htb();
	if(selection == 1)
		goto get_ports;
	transparent_proxy:
	selection = transparent_proxy();
	if(selection == 1)
		goto htb;

	selection = confirm();
	if(selection == 1)
		goto transparent_proxy;
		
    
	/* Clean up */
	destroy_all();
	end();
    
	/* Checking entered date */
	if(name_in == NULL || name_out == NULL || name_in == name_out 
	    || band_in == 0 || band_out == 0 || band_ports == 0)
	{	
		printf("Please enter correct values.\n");
	        exit(1);
	}

	out = fopen(argv[1], "w");
	if(out == NULL)
	{
		printf("Error opening %s file\n", argv[1]);
		printf("Wrong permissions ?\n");
		exit(1);
	}
	
	fprintf(out, "DEV_IN=%s\n", name_in);
	fprintf(out, "DEV_OUT=%s\n", name_out);
	fprintf(out, "BW_IN=%d\n", band_in);
	fprintf(out, "BW_OUT=%d\n", band_out); 
	fprintf(out, "BW_PORTS=%d\n", band_ports);
	if(ports != NULL)
		fprintf(out, "PORTS=\"%s\"\n", ports); 
	else
		fprintf(out, "PORTS=\"\"");
	fprintf(out, "HTB=%c\n", HTB);
	if(proxy != NULL)
	    fprintf(out, "PROXY=\"%s\"\n", proxy);
	else 
	    fprintf(out, "PROXY=\"\"");

	/* Clean up file descriptor */
	fclose(out);
	return 0;
}

